
#include "CustomTreeStore.hh"

CustomTreeStore::CustomTreeStore()
	: Gtk::TreeStore()
{
}

CustomTreeStore::CustomTreeStore( const Gtk::TreeModelColumnRecord& columns )
	: Gtk::TreeStore( columns )
{
}

CustomTreeStore::~CustomTreeStore()
{
}

Glib::RefPtr< CustomTreeStore >
CustomTreeStore::create( const Gtk::TreeModelColumnRecord& columns )
{
	return Glib::RefPtr< CustomTreeStore >( new CustomTreeStore( columns ) ) ;
}

void
CustomTreeStore::reparent( Gtk::TreeModel::iterator& iter, Gtk::TreeModel::iterator& parent )
{
	//Don't try to reparent an iterator to itself.
	if( iter == parent )
		return ;

	//Don't reparent an iterator to a descendant.
	if( is_ancestor( iter, parent ) )
		return ;

	//Start by creating the new branch.
	Gtk::TreeModel::iterator new_iter = append( parent->children() ) ;

	//Copy the desired iterator to the new branch.
	copy_iter( iter, new_iter ) ;

	//Recursively copy from the source branch, to the desination branch.
	recursive_copy( iter->children(), new_iter->children() ) ;

	//Erase the source branch.
	erase( iter ) ;
}

void
CustomTreeStore::copy_iter( Gtk::TreeModel::iterator& from, Gtk::TreeModel::iterator& to )
{
	//Iterator over all columns and copy from one interator to the other.
	for( int i = 0 ; i < get_n_columns() ; i++ )
	{
		Glib::ValueBase val ;

		get_value_impl( from, i, val ) ;
		set_value_impl( to, i, val ) ;
	}
}

void
CustomTreeStore::recursive_copy( Gtk::TreeModel::Children from, Gtk::TreeModel::Children to )
{
	//Copy depth first from one branch to antoher.

	if( from.empty() )
	{
		return ;
	}

	Gtk::TreeModel::iterator iter ;

	for( iter = from.begin() ; iter != from.end() ; iter++ )
	{
		Gtk::TreeModel::iterator f_iter = append( to ) ;

		copy_iter( iter, f_iter ) ;

		recursive_copy( iter->children(), f_iter->children() ) ;
	}
}

