
#include <sstream>
#include <string>

#include "ExampleWindow.hh"

ExampleWindow::ExampleWindow()
{
	set_title( "Custom TreeStore Example." ) ;
	set_default_size( 800, 600 ) ;

	Gtk::VBox* vbox = Gtk::manage( new Gtk::VBox() ) ;

	add( *vbox ) ;

	Gtk::ScrolledWindow* sw = Gtk::manage( new Gtk::ScrolledWindow() ) ;
	sw->set_policy( Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC ) ;
	vbox->pack_start( *sw ) ;

	Gtk::HButtonBox* box = Gtk::manage( new Gtk::HButtonBox() ) ;
	box->set_spacing( 5 ) ;
	box->set_border_width( 5 ) ;
	box->set_layout( Gtk::BUTTONBOX_END ) ;
	vbox->pack_start( *box, Gtk::PACK_SHRINK ) ;

	Gtk::Button* button = Gtk::manage( new Gtk::Button( "Add Row" ) ) ;
	box->pack_start( *button, Gtk::PACK_SHRINK ) ;
	button->signal_clicked().connect( sigc::mem_fun( *this, &ExampleWindow::on_add_row ) ) ;

	button = Gtk::manage( new Gtk::Button( "Quit" ) ) ;
	box->pack_start( *button, Gtk::PACK_SHRINK ) ;
	button->signal_clicked().connect( sigc::mem_fun( *this, &ExampleWindow::on_button_quit ) ) ;

	_store = CustomTreeStore::create( _columns ) ;

	_tree = Gtk::manage( new ExampleView() ) ;
	_tree->set_model( _store ) ;
	_tree->set_reorderable( true ) ;
	_tree->append_column( "Id", _columns.id ) ;
	_tree->append_column( "Text", _columns.text ) ;
	_tree->signal_button_press_event().connect_notify( sigc::mem_fun( *this, &ExampleWindow::before_tree_clicked ) ) ;
	_tree->signal_button_press_event().connect( sigc::mem_fun( *this, &ExampleWindow::on_tree_clicked ) ) ;

	_selection = _tree->get_selection() ;

	sw->add( *_tree ) ;

	show_all_children() ;

	//Non Gui:
	_id = 0 ;
}

ExampleWindow::~ExampleWindow()
{
}

void
ExampleWindow::on_button_quit()
{
	Gtk::Main::quit() ;
}

void
ExampleWindow::on_add_row()
{
	Gtk::TreeModel::Row row = *( _store->append() ) ;

	std::stringstream text ;
	text << "Random Value: " << rand() ;

	row[ _columns.id ]		= _id++ ;
	row[ _columns.text ]	= text.str() ;
}

void
ExampleWindow::before_tree_clicked( GdkEventButton* /*ev*/ )
{
	_from = _selection->get_selected() ;
}

bool
ExampleWindow::on_tree_clicked( GdkEventButton* ev )
{
	if( ev->button != 3 )
		return  true ;

	_parent = _selection->get_selected() ;

	if( _from )
	{
		_store->reparent( _from, _parent ) ;
	}

	return true ;
}
